import org.apache.pulsar.client.api.*;

import java.util.Date;
import java.text.DateFormat;
import java.time.LocalDate;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ExecutionException;

import java.io.IOException;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.stream.Stream;

public class LoadOrder {
	/**
	 * The initiated pulsar client, initiated in the runProducerExample method.
	 */
	private PulsarClient client;

	/**
	 * Constructor
	 */
	LoadOrder() {
	}

	/**
	 * Initiates the pulsar client and sends messages;
	 * 
	 * @throws IOException
	 */
	public void sendMessages(String topicName) throws IOException {
		Map<String, Object> config = getPulsarClientConfig();

		try {
			// Create the pulsar client with the config defined in
			client = PulsarClient.builder().loadConf(config).build();

			// Create a producer for the passed on topic
			Producer<Order> producer = createProducer(topicName);
			Date date = new Date();
			long ts = System.currentTimeMillis();
			DateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
			String stringDate = sdf.format(date);
			System.out.println("Start sending messages for orders at: " + stringDate);

			Stream<Order> records = Files
				//.lines(Paths.get("/Users/erik/i-refactory-demo/tpc_h/deliveredData/Initial load/ORDERS.tbl"))
				.lines(Paths.get("/Users/erik/i-refactory-demo/tpc_h/testData/delivery_00001/ORDERS.tbl"))
				.map( row -> { 
					final Order order = new Order();
					final String[] fields = row.split( "\\|");
		
					order.orderNbr = Integer.parseInt(fields[0]);
					order.customerNbr = Integer.parseInt(fields[1]);
					order.orderStatus = fields[2];
					order.totalPrice = Double.parseDouble(fields[3]);
					order.orderDate = LocalDate.parse(fields[4]);
					order.orderPriority = fields[5];
					order.clerkNbr = fields[6];
					order.shipPriority = Integer.parseInt(fields[7]);
					order.comment = fields[8];
		
					return order;

				});


			int numLines = 0;

			for (Order record : (Iterable<Order>) records::iterator) {
				producer.newMessage()
				.key( "" + record.orderNbr )
				.eventTime(ts)
				.value(record)
				.sendAsync();
				
				numLines++;
				System.out.println("record.orderNbr: " + record.orderNbr);
				if (numLines % 1000000 == 0) {
					System.out.println(sdf.format(new Date()) + " - num lines: " + numLines);
				}
			}

			System.out.println("num lines: " + numLines);

			records.close();
			System.out.println("Waiting for async ops to complete");
			try {
				producer.flushAsync().get();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (ExecutionException e) {
				e.printStackTrace();
			}
			try {
				producer.closeAsync().get();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (ExecutionException e) {
				e.printStackTrace();
			}
			try {
				client.closeAsync().get();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (ExecutionException e) {
				e.printStackTrace();
			}

			date = new Date();
			sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
			stringDate = sdf.format(date);
			System.out.println("All operations completed at: " + stringDate);

		} catch (PulsarClientException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Builds the producer.
	 *
	 * @link https://pulsar.apache.org/api/client/org/apache/pulsar/client/api/ProducerBuilder.html
	 *       for the building methods which can be called to configure the producer.
	 *
	 * @param topicName The name of the topic to which the producer messages' should
	 *                  be send too.
	 *
	 * @return A byte producer.
	 *
	 * @throws PulsarClientException A checked exception which should be catched.
	 */
	private Producer<Order> createProducer(String topicName) throws PulsarClientException {
		Producer<Order> producer = client.newProducer(Schema.JSON(Order.class))
				.topic(topicName)
				.batchingMaxPublishDelay(5000, TimeUnit.MILLISECONDS) 
				.enableBatching(true).batchingMaxMessages(10000).batchingMaxBytes(4000000).blockIfQueueFull(true).maxPendingMessages(20000)
				.compressionType(CompressionType.LZ4).create();

		return producer;
	}

	/**
	 * Get the configuration for the pulsar client.
	 *
	 * @link https://pulsar.apache.org/docs/en/client-libraries-java/#client (for
	 *       configuration values)
	 *
	 * @return A map containing the configuration values for the client.
	 */
	private Map<String, Object> getPulsarClientConfig() {
		Map<String, Object> config = new HashMap<>();

		config.put("serviceUrl", "pulsar://localhost:6650");
		config.put("numIoThreads", 3);
		config.put("numListenerThreads", 3);

		return config;
	}

	public static void main(String[] args) throws IOException {
		LoadOrder producer = new LoadOrder();
		producer.sendMessages("public/default/order-json-key-date");
	}
}




